@@ -39,9 +39,7 @@ module Bundler
       end
 
       def dependencies
-        @dependencies ||= @specs.flat_map do |spec|
-          __dependencies(spec) + metadata_dependencies(spec)
-        end.uniq.sort
+        @dependencies ||= @specs.flat_map(&:expanded_dependencies).uniq.sort
       end
 
       def ==(other)
@@ -71,28 +69,6 @@ module Bundler
       def exemplary_spec
         @specs.first
       end
-
-      def __dependencies(spec)
-        dependencies = []
-        spec.dependencies.each do |dep|
-          next if dep.type == :development
-          dependencies << Dependency.new(dep.name, dep.requirement)
-        end
-        dependencies
-      end
-
-      def metadata_dependencies(spec)
-        [
-          metadata_dependency("Ruby", spec.required_ruby_version),
-          metadata_dependency("RubyGems", spec.required_rubygems_version),
-        ].compact
-      end
-
-      def metadata_dependency(name, requirement)
-        return if requirement.nil? || requirement.none?
-
-        Dependency.new("#{name}\0", requirement)
-      end
     end
   end
 end
