@@ -1,5 +1,5 @@
-# frozen_string_literal: false
-require 'psych/exception'
+# frozen_string_literal: true
+require_relative 'exception'
 
 module Psych
   class SyntaxError < Psych::Exception
