@@ -7,7 +7,7 @@ class RDoc::Markup::Paragraph < RDoc::Markup::Raw
   ##
   # Calls #accept_paragraph on +visitor+
 
-  def accept visitor
+  def accept(visitor)
     visitor.accept_paragraph self
   end
 
@@ -15,7 +15,7 @@ class RDoc::Markup::Paragraph < RDoc::Markup::Raw
   # Joins the raw paragraph text and converts inline HardBreaks to the
   # +hard_break+ text.
 
-  def text hard_break = ''
+  def text(hard_break = '')
     @parts.map do |part|
       if RDoc::Markup::HardBreak === part then
         hard_break
