@@ -1,5 +1,5 @@
 # -*- coding: us-ascii -*-
-# frozen_string_literal: false
+# frozen_string_literal: true
 
 ##
 # A parser is simple a class that subclasses RDoc::Parser and implements #scan
@@ -78,7 +78,7 @@ class RDoc::Parser
 
     return true if s[0, 2] == Marshal.dump('')[0, 2] or s.index("\x00")
 
-    mode = "r"
+    mode = 'r:utf-8' # default source encoding has been changed to utf-8
     s.sub!(/\A#!.*\n/, '')     # assume shebang line isn't longer than 1024.
     encoding = s[/^\s*\#\s*(?:-\*-\s*)?(?:en)?coding:\s*([^\s;]+?)(?:-\*-|[\s;])/, 1]
     mode = "rb:#{encoding}" if encoding
@@ -87,36 +87,11 @@ class RDoc::Parser
     not s.valid_encoding?
   end
 
-  ##
-  # Processes common directives for CodeObjects for the C and Ruby parsers.
-  #
-  # Applies +directive+'s +value+ to +code_object+, if appropriate
-
-  def self.process_directive code_object, directive, value
-    warn "RDoc::Parser::process_directive is deprecated and wil be removed in RDoc 4.  Use RDoc::Markup::PreProcess#handle_directive instead" if $-w
-
-    case directive
-    when 'nodoc' then
-      code_object.document_self = nil # notify nodoc
-      code_object.document_children = value.downcase != 'all'
-    when 'doc' then
-      code_object.document_self = true
-      code_object.force_documentation = true
-    when 'yield', 'yields' then
-      # remove parameter &block
-      code_object.params.sub!(/,?\s*&\w+/, '') if code_object.params
-
-      code_object.block_params = value
-    when 'arg', 'args' then
-      code_object.params = value
-    end
-  end
-
   ##
   # Checks if +file+ is a zip file in disguise.  Signatures from
   # http://www.garykessler.net/library/file_sigs.html
 
-  def self.zip? file
+  def self.zip?(file)
     zip_signature = File.read file, 4
 
     zip_signature == "PK\x03\x04" or
@@ -129,7 +104,7 @@ class RDoc::Parser
   ##
   # Return a parser that can handle a particular extension
 
-  def self.can_parse file_name
+  def self.can_parse(file_name)
     parser = can_parse_by_name file_name
 
     # HACK Selenium hides a jar file using a .txt extension
@@ -142,7 +117,7 @@ class RDoc::Parser
   # Returns a parser that can handle the extension for +file_name+.  This does
   # not depend upon the file being readable.
 
-  def self.can_parse_by_name file_name
+  def self.can_parse_by_name(file_name)
     _, parser = RDoc::Parser.parsers.find { |regexp,| regexp =~ file_name }
 
     # The default parser must not parse binary files
@@ -150,9 +125,11 @@ class RDoc::Parser
     return parser if ext_name.empty?
 
     if parser == RDoc::Parser::Simple and ext_name !~ /txt|rdoc/ then
-      case check_modeline file_name
+      case mode = check_modeline(file_name)
       when nil, 'rdoc' then # continue
-      else return nil
+      else
+        RDoc::Parser.parsers.find { |_, p| return p if mode.casecmp?(p.name[/\w+\z/]) }
+        return nil
       end
     end
 
@@ -163,8 +140,8 @@ class RDoc::Parser
   ##
   # Returns the file type from the modeline in +file_name+
 
-  def self.check_modeline file_name
-    line = open file_name do |io|
+  def self.check_modeline(file_name)
+    line = File.open file_name do |io|
       io.gets
     end
 
@@ -180,14 +157,17 @@ class RDoc::Parser
     return nil if /coding:/i =~ type
 
     type.downcase
-  rescue ArgumentError # invalid byte sequence, etc.
+  rescue ArgumentError
+  rescue Encoding::InvalidByteSequenceError # invalid byte sequence
+
   end
 
   ##
   # Finds and instantiates the correct parser for the given +file_name+ and
   # +content+.
 
-  def self.for top_level, file_name, content, options, stats
+  def self.for(top_level, content, options, stats)
+    file_name = top_level.absolute_name
     return if binary? file_name
 
     parser = use_markup content
@@ -211,7 +191,7 @@ class RDoc::Parser
 
     content = remove_modeline content
 
-    parser.new top_level, file_name, content, options, stats
+    parser.new top_level, content, options, stats
   rescue SystemCallError
     nil
   end
@@ -228,7 +208,7 @@ class RDoc::Parser
   ##
   # Removes an emacs-style modeline from the first line of the document
 
-  def self.remove_modeline content
+  def self.remove_modeline(content)
     content.sub(/\A.*-\*-\s*(.*?\S)\s*-\*-.*\r?\n/, '')
   end
 
@@ -249,7 +229,7 @@ class RDoc::Parser
   #
   # Any comment style may be used to hide the markup comment.
 
-  def self.use_markup content
+  def self.use_markup(content)
     markup = content.lines.first(3).grep(/markup:\s+(\w+)/) { $1 }.first
 
     return unless markup
@@ -272,12 +252,12 @@ class RDoc::Parser
   # RDoc::Markup::PreProcess object is created which allows processing of
   # directives.
 
-  def initialize top_level, file_name, content, options, stats
+  def initialize(top_level, content, options, stats)
     @top_level = top_level
     @top_level.parser = self.class
     @store = @top_level.store
 
-    @file_name = file_name
+    @file_name = top_level.absolute_name
     @content = content
     @options = options
     @stats = stats
@@ -286,15 +266,32 @@ class RDoc::Parser
     @preprocess.options = @options
   end
 
-  autoload :RubyTools, 'rdoc/parser/ruby_tools'
-  autoload :Text,      'rdoc/parser/text'
+  autoload :RubyTools, "#{__dir__}/parser/ruby_tools"
+  autoload :Text,      "#{__dir__}/parser/text"
 
+  ##
+  # Normalizes tabs in +body+
+
+  def handle_tab_width(body)
+    if /\t/ =~ body
+      tab_width = @options.tab_width
+      body.split(/\n/).map do |line|
+        1 while line.gsub!(/\t+/) do
+          b, e = $~.offset(0)
+          ' ' * (tab_width * (e-b) - b % tab_width)
+        end
+        line
+      end.join "\n"
+    else
+      body
+    end
+  end
 end
 
 # simple must come first in order to show up last in the parsers list
-require 'rdoc/parser/simple'
-require 'rdoc/parser/c'
-require 'rdoc/parser/changelog'
-require 'rdoc/parser/markdown'
-require 'rdoc/parser/rd'
-require 'rdoc/parser/ruby'
+require_relative 'parser/simple'
+require_relative 'parser/c'
+require_relative 'parser/changelog'
+require_relative 'parser/markdown'
+require_relative 'parser/rd'
+require_relative 'parser/ruby'
