@@ -1,12 +1,13 @@
 # frozen_string_literal: true
-require_relative '../command'
+
+require_relative "../command"
 
 unless defined? Gem::Commands::MirrorCommand
   class Gem::Commands::MirrorCommand < Gem::Command
     def initialize
-      super('mirror', 'Mirror all gem files (requires rubygems-mirror)')
+      super("mirror", "Mirror all gem files (requires rubygems-mirror)")
       begin
-        Gem::Specification.find_by_name('rubygems-mirror').activate
+        Gem::Specification.find_by_name("rubygems-mirror").activate
       rescue Gem::LoadError
         # no-op
       end
