@@ -209,7 +209,7 @@ module RBS
               super_args = super_class.args
             else
               super_name = BuiltinNames::Object.name
-              super_args = []
+              super_args = [] #: Array[Types::t]
             end
 
             super_name = env.normalize_module_name(super_name)
@@ -219,6 +219,9 @@ module RBS
               InheritModuleError.check!(super_class, env: env)
             end
 
+            super_entry = env.normalized_class_entry(super_name) or raise
+            super_args = AST::TypeParam.normalize_args(super_entry.type_params, super_args)
+
             ancestors = OneAncestors.class_instance(
               type_name: type_name,
               params: params,
@@ -243,9 +246,17 @@ module RBS
 
               module_name = module_self.name
               if module_name.class?
-                module_name = env.normalize_module_name(module_name)
+                module_entry = env.normalized_module_class_entry(module_name) or raise
+                module_name = module_entry.name
+                self_args = AST::TypeParam.normalize_args(module_entry.type_params, module_self.args)
               end
-              self_types.push Definition::Ancestor::Instance.new(name: module_name, args: module_self.args, source: module_self)
+              if module_name.interface?
+                interface_entry = env.interface_decls.fetch(module_name)
+                self_args = AST::TypeParam.normalize_args(interface_entry.decl.type_params, module_self.args)
+              end
+              self_args or raise
+
+              self_types.push Definition::Ancestor::Instance.new(name: module_name, args: self_args, source: module_self)
             end
           end
         end
@@ -346,11 +357,17 @@ module RBS
               MixinClassError.check!(type_name: type_name, env: env, member: member)
               NoMixinFoundError.check!(member.name, env: env, member: member)
 
+              module_decl = env.normalized_module_entry(module_name) or raise
+              module_args = AST::TypeParam.normalize_args(module_decl.type_params, module_args)
+
               module_name = env.normalize_module_name(module_name)
               included_modules << Definition::Ancestor::Instance.new(name: module_name, args: module_args, source: member)
             when member.name.interface? && included_interfaces
               NoMixinFoundError.check!(member.name, env: env, member: member)
 
+              interface_decl = env.interface_decls.fetch(module_name)
+              module_args = AST::TypeParam.normalize_args(interface_decl.decl.type_params, module_args)
+
               included_interfaces << Definition::Ancestor::Instance.new(name: module_name, args: module_args, source: member)
             end
 
@@ -359,8 +376,11 @@ module RBS
               MixinClassError.check!(type_name: type_name, env: env, member: member)
               NoMixinFoundError.check!(member.name, env: env, member: member)
 
-              module_name = env.normalize_module_name(member.name)
+              module_decl = env.normalized_module_entry(member.name) or raise
+              module_name = module_decl.name
+
               module_args = member.args.map {|type| align_params ? type.sub(align_params) : type }
+              module_args = AST::TypeParam.normalize_args(module_decl.type_params, module_args)
 
               prepended_modules << Definition::Ancestor::Instance.new(name: module_name, args: module_args, source: member)
             end
@@ -374,11 +394,17 @@ module RBS
               MixinClassError.check!(type_name: type_name, env: env, member: member)
               NoMixinFoundError.check!(member.name, env: env, member: member)
 
+              module_decl = env.normalized_module_entry(module_name) or raise
+              module_args = AST::TypeParam.normalize_args(module_decl.type_params, module_args)
+
               module_name = env.normalize_module_name(module_name)
               extended_modules << Definition::Ancestor::Instance.new(name: module_name, args: module_args, source: member)
             when member.name.interface? && extended_interfaces
               NoMixinFoundError.check!(member.name, env: env, member: member)
 
+              interface_decl = env.interface_decls.fetch(module_name)
+              module_args = AST::TypeParam.normalize_args(interface_decl.decl.type_params, module_args)
+
               extended_interfaces << Definition::Ancestor::Instance.new(name: module_name, args: module_args, source: member)
             end
           end
@@ -432,7 +458,9 @@ module RBS
             super_name = super_class.name
             super_args = super_class.args
 
-            super_ancestors = instance_ancestors(super_name, building_ancestors: building_ancestors).apply(super_args, location: entry.primary.decl.location)
+            super_ancestors =
+              instance_ancestors(super_name, building_ancestors: building_ancestors)
+                .apply(super_args, env: env, location: entry.primary.decl.super_class&.location)
             super_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: super_name, source: :super) }
             ancestors.unshift(*super_ancestors)
           end
@@ -451,7 +479,10 @@ module RBS
           included_modules.each do |mod|
             name = mod.name
             arg_types = mod.args
-            mod_ancestors = instance_ancestors(name, building_ancestors: building_ancestors).apply(arg_types, location: entry.primary.decl.location)
+            mod.source.is_a?(AST::Members::Include) or raise
+            mod_ancestors =
+              instance_ancestors(name, building_ancestors: building_ancestors)
+                .apply(arg_types, env: env, location: mod.source.location)
             mod_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: name, source: mod.source) }
             ancestors.unshift(*mod_ancestors)
           end
@@ -463,7 +494,10 @@ module RBS
           prepended_modules.each do |mod|
             name = mod.name
             arg_types = mod.args
-            mod_ancestors = instance_ancestors(name, building_ancestors: building_ancestors).apply(arg_types, location: entry.primary.decl.location)
+            mod.source.is_a?(AST::Members::Prepend) or raise
+            mod_ancestors =
+              instance_ancestors(name, building_ancestors: building_ancestors)
+                .apply(arg_types, env: env, location: mod.source.location)
             mod_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: name, source: mod.source) }
             ancestors.unshift(*mod_ancestors)
           end
@@ -491,14 +525,16 @@ module RBS
 
         one_ancestors = one_singleton_ancestors(type_name)
 
-        ancestors = []
+        ancestors = [] #: Array[Definition::Ancestor::t]
 
         case super_class = one_ancestors.super_class
         when Definition::Ancestor::Instance
           super_name = super_class.name
           super_args = super_class.args
 
-          super_ancestors = instance_ancestors(super_name, building_ancestors: building_ancestors).apply(super_args, location: entry.primary.decl.location)
+          super_ancestors =
+            instance_ancestors(super_name, building_ancestors: building_ancestors)
+              .apply(super_args, env: env, location: nil)
           super_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: super_name, source: :super) }
           ancestors.unshift(*super_ancestors)
 
@@ -513,7 +549,10 @@ module RBS
         extended_modules.each do |mod|
           name = mod.name
           args = mod.args
-          mod_ancestors = instance_ancestors(name, building_ancestors: building_ancestors).apply(args, location: entry.primary.decl.location)
+          mod.source.is_a?(AST::Members::Extend) or raise
+          mod_ancestors =
+            instance_ancestors(name, building_ancestors: building_ancestors)
+              .apply(args, env: env, location: mod.source.location)
           mod_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: name, source: mod.source) }
           ancestors.unshift(*mod_ancestors)
         end
@@ -542,11 +581,14 @@ module RBS
         building_ancestors.push self_ancestor
 
         one_ancestors = one_interface_ancestors(type_name)
-        ancestors = []
+        ancestors = [] #: Array[Definition::Ancestor::t]
 
         included_interfaces = one_ancestors.included_interfaces or raise
         included_interfaces.each do |a|
-          included_ancestors = interface_ancestors(a.name, building_ancestors: building_ancestors).apply(a.args, location: entry.decl.location)
+          a.source.is_a?(AST::Members::Include) or raise
+          included_ancestors =
+            interface_ancestors(a.name, building_ancestors: building_ancestors)
+              .apply(a.args, env: env, location: a.source.location)
           included_ancestors.map! {|ancestor| fill_ancestor_source(ancestor, name: a.name, source: a.source) }
           ancestors.unshift(*included_ancestors)
         end
